from pathlib import Path

from aicage.cli_types import ParsedArgs
from aicage.config.context import ConfigContext
from aicage.runtime.docker_args._support._resolver_types import MountRequest, ResolvedArgs


def resolve(
    context: ConfigContext,
    agent: str,
    parsed: ParsedArgs | None,
) -> ResolvedArgs:
    _ = (agent, parsed)
    project_path = Path(context.project_cfg.path).resolve()
    return ResolvedArgs(mounts=[MountRequest(host_path=project_path)])



from aicage._logging import get_logger
from aicage.docker.query import get_local_rootfs_layers


def base_layer_missing(base_image_ref: str, final_image_ref: str) -> bool:
    base_layers = get_local_rootfs_layers(base_image_ref)
    if base_layers is None:
        logger = get_logger()
        logger.warning(
            "Skipping base image layer validation for %s -> %s; missing local layer data.",
            base_image_ref,
            final_image_ref,
        )
        return False
    final_layers = get_local_rootfs_layers(final_image_ref)
    if final_layers is None:
        logger = get_logger()
        logger.warning(
            "Skipping base image layer validation for %s -> %s; missing local layer data.",
            base_image_ref,
            final_image_ref,
        )
        return False
    return base_layers[-1] not in final_layers



from pathlib import Path


def ensure_path_exists(path: Path, is_file: bool) -> Path:
    if path.exists():
        if not path.is_file() and not path.is_dir():
            raise ValueError(f"Path exists but is not a file or directory: {path}")
        return path
    if is_file:
        path.parent.mkdir(parents=True, exist_ok=True)
        path.touch()
    else:
        path.mkdir(parents=True, exist_ok=True)
    return path



import hashlib

from aicage.config.extensions.loader import ExtensionMetadata, extension_hash
from aicage.config.runtime_config import RunConfig
from aicage.docker.build import run_extended_build
from aicage.docker.query import (
    cleanup_old_digest,
    get_local_repo_digest_for_repo,
)
from aicage.docker.refs import repository_from_image_ref
from aicage.registry._build_flow import maybe_build
from aicage.registry._errors import RegistryError
from aicage.registry._time import now_iso

from ._logs import build_log_path
from ._plan import should_rebuild
from ._store import BuildRecord, BuildStore


def ensure(run_config: RunConfig) -> None:
    if not run_config.selection.extensions:
        raise RegistryError("No extensions selected for extended image build.")

    resolved = _resolve_extensions(run_config.selection.extensions, run_config.context.extensions)
    combined_hash = _combined_extension_hash(resolved)
    store = BuildStore()
    image_ref = run_config.selection.image_ref
    maybe_build(
        load_record=lambda: store.load(image_ref),
        should_rebuild=lambda record: should_rebuild(
            run_config=run_config,
            record=record,
            base_image_ref=run_config.selection.base_image_ref,
            extension_hash=combined_hash,
        ),
        run_build=lambda: _run_build(run_config, resolved),
        save_record=lambda: store.save(
            BuildRecord(
                agent=run_config.agent,
                base=run_config.selection.base,
                image_ref=image_ref,
                extensions=list(run_config.selection.extensions),
                extension_hash=combined_hash,
                base_image=run_config.selection.base_image_ref,
                built_at=now_iso(),
            )
        ),
    )


def _run_build(
    run_config: RunConfig,
    resolved: list[ExtensionMetadata],
) -> None:
    image_ref = run_config.selection.image_ref
    image_repository = repository_from_image_ref(image_ref)
    old_digest = get_local_repo_digest_for_repo(image_ref, image_repository)
    log_path = build_log_path(image_ref)
    run_extended_build(
        run_config=run_config,
        base_image_ref=run_config.selection.base_image_ref,
        extensions=resolved,
        log_path=log_path,
    )
    cleanup_old_digest(image_repository, old_digest, image_ref)


def _resolve_extensions(
    extension_ids: list[str],
    extensions: dict[str, ExtensionMetadata],
) -> list[ExtensionMetadata]:
    missing = [ext for ext in extension_ids if ext not in extensions]
    if missing:
        raise RegistryError(f"Missing extensions: {', '.join(sorted(missing))}.")
    return [extensions[ext] for ext in extension_ids]


def _combined_extension_hash(extensions: list[ExtensionMetadata]) -> str:
    digest = hashlib.sha256()
    for extension in extensions:
        digest.update(extension.extension_id.encode("utf-8"))
        digest.update(extension_hash(extension).encode("utf-8"))
    return digest.hexdigest()



import os
import subprocess
from pathlib import Path


def resolve_host_timezone() -> str | None:
    env_timezone = os.environ.get("TZ")
    if env_timezone:
        return env_timezone

    if os.name == "nt":
        return _resolve_windows_timezone()

    return _resolve_posix_timezone()


def _resolve_posix_timezone() -> str | None:
    timezone_file = Path("/etc/timezone")
    if timezone_file.is_file():
        timezone = timezone_file.read_text(encoding="utf-8").strip()
        if timezone:
            return timezone

    localtime_path = Path("/etc/localtime")
    if not localtime_path.exists():
        return None

    resolved = localtime_path.resolve()
    parts = resolved.parts
    if "zoneinfo" not in parts:
        return None

    zoneinfo_index = parts.index("zoneinfo")
    timezone_parts = parts[zoneinfo_index + 1 :]
    if not timezone_parts:
        return None

    return "/".join(timezone_parts)


def _resolve_windows_timezone() -> str | None:
    command = (
        "[System.TimeZoneInfo]::TryConvertWindowsIdToIanaId("
        "[System.TimeZoneInfo]::Local.Id, [ref]$iana) | Out-Null; "
        "$iana"
    )
    for shell in ("powershell", "pwsh"):
        try:
            result = subprocess.run(
                [shell, "-NoProfile", "-Command", command],
                check=False,
                capture_output=True,
                text=True,
            )
        except OSError:
            continue
        timezone = result.stdout.strip()
        if result.returncode == 0 and timezone:
            return timezone
    return None



from collections.abc import Mapping
from pathlib import Path

from aicage.cli_types import ParsedArgs
from aicage.config.context import ConfigContext
from aicage.config.extensions.loader import ExtensionMetadata
from aicage.config.project_config import AgentConfig
from aicage.runtime.docker_args._support._resolver_types import MountRequest, ResolvedArgs
from aicage.runtime.mounts.shares import ShareSpec, resolve_share_specs


def resolve(
    context: ConfigContext,
    agent: str,
    parsed: ParsedArgs | None,
) -> ResolvedArgs:
    _ = (context, agent)
    if parsed is None:
        return ResolvedArgs()
    cwd = Path.cwd().resolve()
    agent_cfg: AgentConfig = context.project_cfg.agents.setdefault(agent, AgentConfig())
    extension_shares = _approved_extension_shares(agent_cfg, context.extensions)
    share_mounts: list[ShareSpec] = resolve_share_specs([*parsed.shares, *extension_shares], cwd)
    mounts = [MountRequest(host_path=share.host_path, read_only=share.read_only) for share in share_mounts]
    return ResolvedArgs(mounts=mounts)


def _approved_extension_shares(
    agent_cfg: AgentConfig,
    available_extensions: Mapping[str, ExtensionMetadata],
) -> list[str]:
    shares: list[str] = []
    for extension_id in agent_cfg.extensions:
        if not agent_cfg.extension_mounts.get(extension_id):
            continue
        extension = available_extensions.get(extension_id)
        if extension is None:
            continue
        shares.extend(extension.shares)
    return shares



import socket
import ssl
import urllib.error
from urllib.parse import urlparse

_HTTP_PROXY_AUTH_REQUIRED: int = 407
_HTTP_AUTH_STATUS_CODES: set[int] = {401, 403}


def classify_network_failure(exc: BaseException) -> str:
    category = "unknown"
    if isinstance(exc, urllib.error.HTTPError):
        if exc.code == _HTTP_PROXY_AUTH_REQUIRED:
            category = "proxy_auth_407"
        elif exc.code in _HTTP_AUTH_STATUS_CODES:
            category = "auth_401_403"
        else:
            category = "http_error"
    elif isinstance(exc, urllib.error.URLError):
        category = _classify_url_error_reason(exc.reason)
    elif isinstance(exc, socket.gaierror):
        category = "dns"
    elif isinstance(exc, (socket.timeout, TimeoutError)):
        category = "connect_or_timeout"
    elif isinstance(exc, ssl.SSLCertVerificationError):
        category = "tls"
    elif isinstance(exc, ssl.SSLError):
        category = "tls"
    elif isinstance(exc, OSError):
        category = "connect_or_timeout"
    return category


def host_from_url(url: str) -> str:
    parsed = urlparse(url)
    if parsed.hostname:
        return parsed.hostname
    return url


def _classify_url_error_reason(reason: object) -> str:
    if isinstance(reason, socket.gaierror):
        return "dns"
    if isinstance(reason, ssl.SSLCertVerificationError):
        return "tls"
    if isinstance(reason, ssl.SSLError):
        return "tls"
    if isinstance(reason, (socket.timeout, TimeoutError)):
        return "connect_or_timeout"
    if isinstance(reason, OSError):
        return "connect_or_timeout"
    return "unknown"



from dataclasses import dataclass, field
from pathlib import Path
from typing import Protocol

from aicage.cli_types import ParsedArgs
from aicage.config.context import ConfigContext
from aicage.runtime.run_args import EnvVar


@dataclass(frozen=True)
class MountRequest:
    host_path: Path
    read_only: bool = False


@dataclass(frozen=True)
class ResolvedArgs:
    mounts: list[MountRequest] = field(default_factory=list)
    env: list[EnvVar] = field(default_factory=list)


class Resolver(Protocol):
    def __call__(
        self,
        context: ConfigContext,
        agent: str,
        parsed: ParsedArgs | None,
    ) -> ResolvedArgs:
        ...



from pathlib import Path

from aicage._logging import get_logger
from aicage.config.agent.models import AgentMetadata
from aicage.config.runtime_config import RunConfig
from aicage.docker.build import run_build
from aicage.docker.query import (
    cleanup_old_digest,
    get_local_repo_digest_for_repo,
    local_image_exists,
)
from aicage.docker.refs import repository_from_image_ref
from aicage.paths import CUSTOM_BASES_DIR
from aicage.registry._build_flow import maybe_build
from aicage.registry._errors import RegistryError
from aicage.registry._time import now_iso

from ..base_build.ensure import ensure as ensure_base_build
from ._digest import refresh_base_digest
from ._logs import build_log_path
from ._plan import should_rebuild
from ._refs import base_repository, get_base_image_ref
from ._store import BuildRecord, BuildStore
from .agent_version.checker import AgentVersionChecker


def ensure(run_config: RunConfig) -> None:
    agent_metadata = run_config.context.agents[run_config.agent]
    definition_dir = agent_metadata.local_definition_dir

    base_metadata = run_config.context.bases[run_config.selection.base]
    custom_base = base_metadata.local_definition_dir.is_relative_to(CUSTOM_BASES_DIR)
    base_image = get_base_image_ref(run_config)
    image_ref = run_config.selection.base_image_ref
    if custom_base:
        ensure_base_build(
            run_config.selection.base,
            base_metadata,
            base_metadata.local_definition_dir,
        )
    else:
        base_repo = base_repository(run_config)
        try:
            base_image = refresh_base_digest(
                base_image_ref=base_image,
                base_repository=base_repo,
            )
        except RegistryError:
            if not local_image_exists(image_ref):
                raise
            logger = get_logger()
            logger.warning(
                "Base image refresh failed; using local agent image %s.",
                image_ref,
            )
            return

    store = BuildStore()
    agent_version = _get_agent_version(run_config, agent_metadata, definition_dir)
    maybe_build(
        load_record=lambda: store.load(run_config.agent, run_config.selection.base),
        should_rebuild=lambda record: should_rebuild(
            run_config=run_config,
            record=record,
            agent_version=agent_version,
            base_image_ref=base_image,
        ),
        run_build=lambda: _run_build(run_config, base_image, image_ref),
        save_record=lambda: store.save(
            BuildRecord(
                agent=run_config.agent,
                base=run_config.selection.base,
                agent_version=agent_version,
                base_image=base_image,
                image_ref=image_ref,
                built_at=now_iso(),
            )
        ),
    )


def _get_agent_version(
    run_config: RunConfig,
    agent_metadata: AgentMetadata,
    definition_dir: Path,
) -> str:
    checker = AgentVersionChecker()
    return checker.get_version(
        run_config.agent,
        agent_metadata,
        definition_dir,
    )


def _run_build(
    run_config: RunConfig,
    base_image_ref: str,
    image_ref: str,
) -> None:
    image_repository = repository_from_image_ref(image_ref)
    old_digest = get_local_repo_digest_for_repo(image_ref, image_repository)
    log_path = build_log_path(run_config.agent, run_config.selection.base)
    run_build(
        run_config=run_config,
        base_image_ref=base_image_ref,
        image_ref=image_ref,
        log_path=log_path,
    )
    cleanup_old_digest(image_repository, old_digest, image_ref)



from aicage.config.agent._loader_shared import load_agents_from_directory
from aicage.config.agent.models import AgentMetadata
from aicage.config.base.models import BaseMetadata
from aicage.paths import CUSTOM_AGENT_DEFINITION_FILES, CUSTOM_AGENTS_DIR


def load_custom_agents(
    bases: dict[str, BaseMetadata],
) -> dict[str, AgentMetadata]:
    agents_dir = CUSTOM_AGENTS_DIR
    if not agents_dir.is_dir():
        return {}
    return load_agents_from_directory(
        agents_dir=agents_dir,
        bases=bases,
        definition_files=CUSTOM_AGENT_DEFINITION_FILES,
        agent_label="Custom agent",
    )



from pathlib import Path

from aicage.config.agent._metadata import build_agent_metadata
from aicage.config.agent._validation import ensure_required_files
from aicage.config.agent.models import AgentMetadata
from aicage.config.base.models import BaseMetadata
from aicage.config.errors import ConfigError
from aicage.config.yaml_loader import load_yaml


def load_agents_from_directory(
    agents_dir: Path,
    bases: dict[str, BaseMetadata],
    definition_files: tuple[str, ...],
    agent_label: str,
) -> dict[str, AgentMetadata]:
    agents: dict[str, AgentMetadata] = {}
    for entry in sorted(agents_dir.iterdir()):
        if not entry.is_dir():
            continue
        agent_name = entry.name
        agent_path = _find_agent_definition(entry, definition_files, agent_label)
        agent_mapping = load_yaml(agent_path)
        ensure_required_files(agent_name, entry)
        agents[agent_name] = build_agent_metadata(
            agent_name=agent_name,
            agent_mapping=agent_mapping,
            bases=bases,
            definition_dir=entry,
        )
    return agents


def _find_agent_definition(agent_dir: Path, definition_files: tuple[str, ...], agent_label: str) -> Path:
    for filename in definition_files:
        candidate = agent_dir / filename
        if candidate.is_file():
            return candidate
    expected = ", ".join(definition_files)
    raise ConfigError(f"{agent_label} '{agent_dir.name}' is missing {expected}.")



from pathlib import Path

from aicage.config.base.models import BaseMetadata
from aicage.constants import LOCAL_IMAGE_BASE_REPOSITORY
from aicage.docker.build import run_custom_base_build
from aicage.docker.query import (
    cleanup_old_digest,
    get_local_repo_digest_for_repo,
    local_image_exists,
)
from aicage.registry._build_flow import maybe_build
from aicage.registry._time import now_iso
from aicage.registry.digest.remote_digest import get_remote_digest

from ._logs import build_log_path
from ._store import BuildRecord, BuildStore


def image_ref(base: str) -> str:
    return f"{LOCAL_IMAGE_BASE_REPOSITORY}:{base}"


def ensure(base: str, base_metadata: BaseMetadata, base_dir: Path) -> None:
    target_image_ref = image_ref(base)
    local_exists = local_image_exists(target_image_ref)
    store = BuildStore()
    source_digest = get_remote_digest(base_metadata.from_image)

    maybe_build(
        load_record=lambda: store.load(base),
        should_rebuild=lambda record: _should_rebuild(
            local_exists=local_exists,
            record=record,
            base_metadata=base_metadata,
            source_digest=source_digest,
        ),
        run_build=lambda: _run_build(
            base=base,
            base_metadata=base_metadata,
            base_dir=base_dir,
            target_image_ref=target_image_ref,
        ),
        save_record=lambda: store.save(
            BuildRecord(
                base=base,
                from_image=base_metadata.from_image,
                from_image_digest=source_digest or "",
                image_ref=target_image_ref,
                built_at=now_iso(),
            )
        ),
    )


def _should_rebuild(
    local_exists: bool,
    record: BuildRecord | None,
    base_metadata: BaseMetadata,
    source_digest: str | None,
) -> bool:
    if not local_exists or record is None:
        return True
    if record.from_image != base_metadata.from_image:
        return True
    if source_digest and record.from_image_digest != source_digest:
        return True
    return False


def _run_build(
    base: str,
    base_metadata: BaseMetadata,
    base_dir: Path,
    target_image_ref: str,
) -> None:
    old_digest = get_local_repo_digest_for_repo(target_image_ref, LOCAL_IMAGE_BASE_REPOSITORY)
    log_path = build_log_path(base)
    run_custom_base_build(
        dockerfile_path=base_dir / "Dockerfile",
        build_root=base_dir,
        from_image=base_metadata.from_image,
        image_ref=target_image_ref,
        log_path=log_path,
    )
    cleanup_old_digest(LOCAL_IMAGE_BASE_REPOSITORY, old_digest, target_image_ref)



from pathlib import Path

from aicage.paths import container_project_path
from aicage.runtime.docker_args._support._resolver_types import MountRequest
from aicage.runtime.run_args import MountSpec


def map_mount_requests(requests: list[MountRequest]) -> list[MountSpec]:
    deduped_requests = _dedupe_exact_mount_requests(requests)
    # Process shallower paths first so parents are seen before children.
    # That makes nested deduplication one-directional.
    sorted_requests = _sort_mount_requests(deduped_requests)
    selected_requests = _dedupe_nested_mount_requests(sorted_requests)
    return _build_mount_specs(selected_requests)


def _dedupe_exact_mount_requests(requests: list[MountRequest]) -> list[MountRequest]:
    deduped_requests: dict[Path, MountRequest] = {}
    for request in requests:
        host_path = request.host_path.resolve()
        if host_path in deduped_requests:
            continue
        deduped_requests[host_path] = MountRequest(host_path=host_path, read_only=request.read_only)
    return list(deduped_requests.values())


def _dedupe_nested_mount_requests(requests: list[MountRequest]) -> list[MountRequest]:
    selected_requests: list[MountRequest] = []
    for request in requests:
        host_path = request.host_path.resolve()
        if any(
            _is_redundant_nested_mount(request, selected, host_path, selected.host_path.resolve())
            for selected in selected_requests
        ):
            continue
        selected_requests.append(request)
    return selected_requests


def _build_mount_specs(requests: list[MountRequest]) -> list[MountSpec]:
    mounts: list[MountSpec] = []
    for request in requests:
        host_path = request.host_path.resolve()
        container_path = container_project_path(host_path)
        mounts.append(
            MountSpec(
                host_path=host_path,
                container_path=container_path,
                read_only=request.read_only,
            )
        )
    return mounts


def _sort_mount_requests(requests: list[MountRequest]) -> list[MountRequest]:
    return sorted(
        requests,
        key=lambda request: (
            len(request.host_path.resolve().parts),
            request.host_path.resolve().as_posix(),
        ),
    )


def _is_redundant_nested_mount(
    nested_request: MountRequest,
    parent_request: MountRequest,
    nested_host_path: Path,
    parent_host_path: Path,
) -> bool:
    return (
        nested_request.read_only == parent_request.read_only
        and _is_path_within(nested_host_path, parent_host_path)
    )


def _is_path_within(path: Path, parent: Path) -> bool:
    try:
        path.relative_to(parent)
        return True
    except ValueError:
        return False



from dataclasses import dataclass


@dataclass
class ParsedArgs:
    dry_run: bool
    docker_args: str
    agent: str
    agent_args: list[str]
    docker_socket: bool
    shares: list[str]
    config_action: str | None
    config_agent: str | None = None
    yes: bool = False



from aicage._proxy import proxy_env_vars_from_host
from aicage.cli_types import ParsedArgs
from aicage.config.runtime_config import RunConfig
from aicage.runtime._host_timezone import resolve_host_timezone
from aicage.runtime.run_args import DockerRunArgs, EnvVar, merge_docker_args


def build_run_args(config: RunConfig, parsed: ParsedArgs) -> DockerRunArgs:
    merged_docker_args: str = merge_docker_args(
        config.project_docker_args,
        parsed.docker_args,
    )
    return DockerRunArgs(
        image_ref=config.selection.image_ref,
        merged_docker_args=merged_docker_args,
        agent_args=parsed.agent_args,
        env=[*config.env, *_host_timezone_env(config.env), *proxy_env_vars_from_host()],
        mounts=list(config.mounts),
    )


def _host_timezone_env(existing_env: list[EnvVar]) -> list[EnvVar]:
    if any(env.name == "TZ" for env in existing_env):
        return []

    timezone = resolve_host_timezone()
    if not timezone:
        return []

    return [EnvVar(name="TZ", value=timezone)]



from aicage.config._schema_validation import load_schema, validate_schema_mapping

_EXTENSION_SCHEMA_PATH = "validation/extension.schema.json"
_EXTENSION_CONTEXT = "extension metadata"


def validate_extension_mapping(mapping: dict[str, object]) -> dict[str, object]:
    schema = load_schema(_EXTENSION_SCHEMA_PATH)
    return validate_schema_mapping(mapping, schema, _EXTENSION_CONTEXT)



import json
import re
import subprocess
import urllib.error
import urllib.request

from aicage._logging import get_logger
from aicage._network import classify_network_failure, host_from_url
from aicage.constants import PYPI_VERSION_CHECK_TIMEOUT_SECONDS
from aicage.runtime.prompts.confirm import prompt_update_aicage

_PYPI_URL: str = "https://pypi.org/pypi/aicage/json"
_UPGRADE_COMMAND: str = "pipx upgrade aicage"
_UNKNOWN_VERSION: str = "0.0.0"


def _check_for_update(current_version: str) -> str | None:
    logger = get_logger()
    host = host_from_url(_PYPI_URL)
    try:
        request = urllib.request.Request(_PYPI_URL, headers={"Accept": "application/json"})
        with urllib.request.urlopen(request, timeout=PYPI_VERSION_CHECK_TIMEOUT_SECONDS) as response:
            payload = json.loads(response.read().decode("utf-8"))
    except (urllib.error.URLError, TimeoutError) as exc:
        logger.warning(
            "Version check failed (operation=pypi_version_check, host=%s, category=%s): %s",
            host,
            classify_network_failure(exc),
            exc,
        )
        return None
    except json.JSONDecodeError as exc:
        logger.warning("Version check failed (operation=pypi_version_check, host=%s): %s", host, exc)
        return None

    latest_version = str(payload.get("info", {}).get("version", "")).strip()
    if not latest_version:
        return None

    if _is_newer(latest_version, current_version):
        return latest_version

    return None


def maybe_prompt_update(current_version: str) -> bool:
    if current_version == _UNKNOWN_VERSION:
        return False
    latest_version = _check_for_update(current_version)
    if not latest_version:
        return False

    if prompt_update_aicage(current_version, latest_version):
        return _run_upgrade()
    print(f"Update with: {_UPGRADE_COMMAND}")
    return False


def _run_upgrade() -> bool:
    logger = get_logger()
    try:
        result = subprocess.run(
            _UPGRADE_COMMAND.split(),
            check=False,
            capture_output=True,
            text=True,
        )
    except FileNotFoundError as exc:
        logger.error("Upgrade command failed: %s", exc)
        print(f"Unable to run '{_UPGRADE_COMMAND}'. Please run it manually.")
        return False

    if result.returncode != 0:
        logger.error(
            "Upgrade command failed (%s): %s",
            result.returncode,
            result.stderr.strip(),
        )
        print(f"Upgrade failed. Please run '{_UPGRADE_COMMAND}' manually.")
        return False
    return True


def _is_newer(latest_version: str, current_version: str) -> bool:
    latest = _parse_version(latest_version)
    current = _parse_version(current_version)
    if latest is None or current is None:
        return False
    length = max(len(latest), len(current))
    latest_padded = latest + (0,) * (length - len(latest))
    current_padded = current + (0,) * (length - len(current))
    return latest_padded > current_padded


def _parse_version(version: str) -> tuple[int, ...] | None:
    match = re.match(r"^(\d+(?:\.\d+)*)", version)
    if not match:
        return None
    return tuple(int(part) for part in match.group(1).split("."))



from pathlib import Path
from typing import Any

from aicage.config._schema_validation import load_schema, validate_schema_mapping
from aicage.config.agent.models import (
    AGENT_PATH_DIRECTORIES_KEY,
    AGENT_PATH_FILES_KEY,
    AGENT_PATH_KEY,
    BUILD_LOCAL_KEY,
)
from aicage.config.errors import ConfigError

_AGENT_SCHEMA_PATH = "validation/agent.schema.json"
_AGENT_CONTEXT = "agent metadata"


def validate_agent_mapping(mapping: dict[str, Any]) -> dict[str, Any]:
    schema = load_schema(_AGENT_SCHEMA_PATH)
    return validate_schema_mapping(
        mapping,
        schema,
        _AGENT_CONTEXT,
        normalizer=_apply_defaults,
    )


def ensure_required_files(agent_name: str, agent_dir: Path) -> None:
    missing = [name for name in ("install.sh", "version.sh") if not (agent_dir / name).is_file()]
    if missing:
        raise ConfigError(f"Agent '{agent_name}' is missing {', '.join(missing)}.")


def _apply_defaults(mapping: dict[str, Any]) -> dict[str, Any]:
    normalized = dict(mapping)
    normalized.setdefault(BUILD_LOCAL_KEY, True)
    agent_path = normalized.get(AGENT_PATH_KEY)
    if isinstance(agent_path, dict):
        agent_path = dict(agent_path)
        agent_path.setdefault(AGENT_PATH_FILES_KEY, [])
        agent_path.setdefault(AGENT_PATH_DIRECTORIES_KEY, [])
        normalized[AGENT_PATH_KEY] = agent_path
    return normalized
